<!---
I'm a simple XML-based charting engine.  The descriptor XML may be
passed either as the body of the tag, or via the 'descriptor' attribute.
See ChartViewer.mxml for details of the descriptor XML.

NOTE: This file MUST reside in a web accessible location, along with
ChartViewer.mxml, as the compiled SWF is placed into the same directory,
and requested directly.  If invoked directly (i.e., not as a custom tag)
the file will simply exit with no output.

This tag doesn't expose some of the more advanced capabilities of
ChartViewer.mxml, such as runtime descriptor swapping and URL-based
descriptor fetching, though all but a FlashVars-configured descriptorUrl
are doable via your own means.
--->
<cfimport prefix="mm" taglib="/WEB-INF/lib/cf-bootstrap-for-flex.jar" />

<cfif NOT structKeyExists(variables, "thisTag")>
	<cfexit method="exittemplate" />
</cfif>

<cfparam name="attributes.width" default="450" />
<cfparam name="attributes.height" default="300" />
<cfparam name="attributes.id" default="chart_#getTickCount()#_#removeChars(rand(), 1, 2)#" />
<cfparam name="attributes.baseUrl" default="" />

<cffunction name="drawChart" output="true" returntype="void">
	<cfset var rootPath = REReplace(replace(expandPath("."), "\", "/", "all"), "/+$", "") />
	<cfset var thisPath = REReplace(replace(getDirectoryFromPath(getCurrentTemplatePath()), "\", "/", "all"), "/+$", "") />
	<cfset var relPath = relativePath(rootPath, thisPath) />
	<cfif NOT isXml(attributes.descriptor)>
		<cfthrow type="InvalidDataException"
			message="The provided descriptor is not XML." />
	</cfif>
	<mm:mxml id="#attributes.id#"
		width="#attributes.width#"
		height="#attributes.height#"
		source="#relPath#ChartViewer.mxml">
		<mm:flashvar name="descriptor" value="#attributes.descriptor#" />
		<cfif len(trim(attributes.baseUrl)) GT 0>
			<mm:flashvar name="baseUrl" value="#attributes.baseUrl#" />
		</cfif>
	</mm:mxml>
</cffunction>

<cffunction name="relativePath" returntype="string" access="public" output="false"
			hint="I compute the relative path from one file system location to another.">
	<cfargument name="from" type="string" required="true"
				hint="I am the full pathname from which the relative path should be computed." />
	<cfargument name="to" type="string" required="true"
				hint="I am the full pathname to which the relative path should be computed." />

	<cfset var relative = "" />
	<cfset var fromFirst = listFirst(arguments.from,"/") />
	<cfset var fromRest = arguments.from />
	<cfset var toFirst = listFirst(arguments.to,"/") />
	<cfset var toRest = arguments.to />
	<cfset var needSlash = false />

	<!--- trap special case first --->
	<cfif arguments.from is arguments.to>
		<cfreturn "" />
	</cfif>

	<!--- walk down the common part of the paths --->
	<cfloop condition="fromFirst is toFirst">
		<cfset needSlash = true />
		<cfset fromRest = listRest(fromRest,"/") />
		<cfset toRest = listRest(toRest,"/") />
		<cfset fromFirst = listFirst(fromRest,"/") />
		<cfset toFirst = listFirst(toRest,"/") />
	</cfloop>
	<!--- at this point the paths differ --->
	<cfif not needSlash>
		<!--- the paths differed from the top so we need to strip the leading / --->
		<cfset toRest = right(toRest,len(toRest)-1) />
	</cfif>
	<cfset relative = repeatString("../",listLen(fromRest,"/")) & toRest />
	<!---
		ensure the trailing / is present - strictly speaking this is a bug fix for Railo
		but it's probably a good practice anyway
	--->
	<cfif right(relative,1) is not "/">
		<cfset relative = relative & "/" />
	</cfif>

	<cfreturn relative />

</cffunction>

<cfif thisTag.executionMode EQ "start">
	<cfif NOT thisTag.hasEndTag>
		<cfif NOT structKeyExists(attributes, "descriptor")>
			<cfthrow type="NoDataException"
				message="If you don't provide a descriptor in the tag body, you must provide it via the 'descriptor' attribute." />
		</cfif>
		<cfset drawChart() />
	</cfif>
<cfelse> <!--- end --->
	<cfset attributes.descriptor = thisTag.generatedContent />
	<cfset thisTag.generatedContent = "" />
	<cfset drawChart() />
</cfif>